using System;
using System.Collections;
using Server;
using Server.Items;
using Server.Targeting;
using Server.Targets;
using Server.Network;
using Server.ContextMenus;

namespace Server.Mobiles
{
	public enum AIType
	{
		AI_Use_Default,
		AI_Melee,
		AI_Animal,
		AI_Archer,
		AI_Healer,
		AI_Vendor,
		AI_Mage,
		AI_Berserk,
		AI_Predator,
		AI_Thief
	}

	public enum ActionType
	{
		Wander,
		Combat,
		Guard,
		Flee,
		Backoff,
		Interact
	}

	public abstract class BaseAI
	{
		public Timer			m_Timer;
		protected ActionType	m_Action;
		private DateTime		m_NextStopGuard;
		private Mobile			m_AutoTarget;

		public BaseCreature m_Mobile;

		public BaseAI(BaseCreature m)
		{
			m_Mobile = m;

			m_Timer = new AITimer( this );
			m_Timer.Start();

			m_Mobile.WaitForTarget = false;

			Action = ActionType.Wander;
		}

		public ActionType Action
		{
			get
			{
				return m_Action;
			}
			set
			{
				m_Action = value;
				OnActionChanged();
			}
		}

		public virtual bool WasNamed( string speech )
		{
			string name = m_Mobile.Name;

			if ( name == null || name.Length > speech.Length )
				return false;

			return ( CaseInsensitiveComparer.Default.Compare( name, speech.Substring( 0, name.Length ) ) == 0 );
		}

		private class InternalEntry : ContextMenuEntry
		{
			private BaseCreature m_Mobile;
			private BaseAI m_AI;
			private OrderType m_Order;

			public InternalEntry( int number, int range, BaseCreature mobile, BaseAI ai, OrderType order ) : base( number, range )
			{
				m_Mobile = mobile;
				m_AI = ai;
				m_Order = order;
			}

			public override void OnClick()
			{
				if ( !m_Mobile.Deleted && m_Mobile.Controled && Owner.From == m_Mobile.ControlMaster )
				{
					m_AI.m_AutoTarget = null;
					m_Mobile.ControlOrder = m_Order;
				}
			}
		}

		public virtual void GetContextMenuEntries( Mobile from, ArrayList list )
		{
			if ( m_Mobile.Controled && from == m_Mobile.ControlMaster && from.InRange( m_Mobile, 14 ) )
			{
				list.Add( new InternalEntry( 6107, 14, m_Mobile, this, OrderType.Guard ) );  // Command: Guard
				list.Add( new InternalEntry( 6108, 14, m_Mobile, this, OrderType.Follow ) ); // Command: Follow

				if ( !m_Mobile.Summoned )
					list.Add( new InternalEntry( 6109, 14, m_Mobile, this, OrderType.Drop ) );   // Command: Drop

				list.Add( new InternalEntry( 6111, 14, m_Mobile, this, OrderType.Attack ) ); // Command: Kill
				list.Add( new InternalEntry( 6112, 14, m_Mobile, this, OrderType.Stop ) );   // Command: Stop
				list.Add( new InternalEntry( 6114, 14, m_Mobile, this, OrderType.Stay ) );   // Command: Stay
			}
		}

		public virtual void OnSpeech( SpeechEventArgs e )
		{
			if ( m_Mobile.Controled )
			{
				m_Mobile.DebugSay("I listen");

				if ( e.Mobile == m_Mobile.ControlMaster )
				{
					m_Mobile.DebugSay("Its from my master");

					int[] keywords = e.Keywords;
					string speech = e.Speech;

					m_AutoTarget = null;

					// First, check the all*
					for ( int i = 0; i < keywords.Length; ++i )
					{
						int keyword = keywords[i];

						switch ( keyword )
						{
							case 0x164: // all come
							{
								m_Mobile.ControlOrder = OrderType.Come;
								return;
							}
							case 0x165: // all follow
							{
								m_Mobile.ControlOrder = OrderType.Follow;
								return;
							}
							case 0x166: // all guard
							{
								m_Mobile.ControlOrder = OrderType.Guard;
								return;
							}
							case 0x167: // all stop
							{
								m_Mobile.ControlOrder = OrderType.Stop;
								return;
							}
							case 0x168: // all kill
							case 0x169: // all attack
							{
								m_Mobile.ControlOrder = OrderType.Attack;
								return;
							}
							case 0x16B: // all guard me
							{
								m_AutoTarget = e.Mobile;
								m_Mobile.ControlOrder = OrderType.Guard;
								return;
							}
							case 0x16C: // all follow me
							{
								m_AutoTarget = e.Mobile;
								m_Mobile.ControlOrder = OrderType.Follow;
								return;
							}
							case 0x170: // all stay
							{
								m_Mobile.ControlOrder = OrderType.Stay;
								return;
							}
						}
					}

					// No all*, so check *command
					for ( int i = 0; i < keywords.Length; ++i )
					{
						int keyword = keywords[i];

						switch ( keyword )
						{
							case 0x155: // *come
							{
								if ( WasNamed( speech ) )
									m_Mobile.ControlOrder = OrderType.Come;

								return;
							}
							case 0x156: // *drop
							{
								if ( !m_Mobile.Summoned && WasNamed( speech ) )
									m_Mobile.ControlOrder = OrderType.Drop;

								return;
							}
							case 0x15A: // *follow
							{
								if ( WasNamed( speech ) )
									m_Mobile.ControlOrder = OrderType.Follow;

								return;
							}
							case 0x15C: // *guard
							{
								if ( WasNamed( speech ) )
									m_Mobile.ControlOrder = OrderType.Guard;

								return;
							}
							case 0x15D: // *kill
							case 0x15E: // *attack
							{
								if ( WasNamed( speech ) )
									m_Mobile.ControlOrder = OrderType.Attack;

								return;
							}
							case 0x15F: // *patrol
							{
								if ( WasNamed( speech ) )
									m_Mobile.ControlOrder = OrderType.Patrol;

								return;
							}
							case 0x161: // *stop
							{
								if ( WasNamed( speech ) )
									m_Mobile.ControlOrder = OrderType.Stop;

								return;
							}
							case 0x163: // *follow me
							{
								if ( WasNamed( speech ) )
								{
									m_AutoTarget = e.Mobile;
									m_Mobile.ControlOrder = OrderType.Follow;
								}

								return;
							}
							case 0x16D: // *release
							{
								if ( WasNamed( speech ) )
									m_Mobile.ControlOrder = OrderType.Release;

								return;
							}
							case 0x16E: // *transfer
							{
								if ( WasNamed( speech ) )
								{
									if ( m_Mobile.Summoned )
										e.Mobile.SendLocalizedMessage( 1005487 ); // You cannot transfer ownership of a summoned creature.
									else
										m_Mobile.ControlOrder = OrderType.Transfert;
								}

								return;
							}
							case 0x16F: // *stay
							{
								if ( WasNamed( speech ) )
									m_Mobile.ControlOrder = OrderType.Stay;

								return;
							}
						}
					}

					/*// SkillCheck here.  Can be a exploit for rising Animal Taming.  Need to be fixed.
						
					if ( m_Mobile.ControlMaster.AccessLevel < AccessLevel.GameMaster && !m_Mobile.Summoned && !e.Mobile.CheckSkill( SkillName.AnimalTaming, m_Mobile.MinTameSkill, m_Mobile.MaxTameSkill ) )
					{
						m_Mobile.PlaySound( m_Mobile.GetAngerSound() );
						return;
					}*/
				}
			}
			else
			{
				if ( e.Mobile.AccessLevel >= AccessLevel.GameMaster )
				{
					m_Mobile.DebugSay("Its from a GM");

					if ( m_Mobile.FindMyName( e.Speech, true ) )
					{
						string[] str = e.Speech.Split(' ');
						int i;

						for (i=0 ; i < str.Length; i++)
						{
							string word = str[i];
							
							if ( word.ToLower() == "obey" )
							{
								m_Mobile.SetControlMaster( e.Mobile );
								return;
							}
						}
					}
				}
			}
		}

		public virtual bool Think()
		{
			if ( m_Mobile.Deleted )
				return false;

			switch (Action)
			{
				case ActionType.Wander:
					return DoActionWander();

				case ActionType.Combat:
					return DoActionCombat();

				case ActionType.Guard:
					return DoActionGuard();

				case ActionType.Flee:
					return DoActionFlee();

				case ActionType.Interact:
					return DoActionInteract();

				case ActionType.Backoff:
					return DoActionBackoff();

				default:
					return false;
			}
		}

		public virtual void OnActionChanged()
		{
			switch (Action)
			{
				case ActionType.Wander:
					m_Mobile.Warmode = false;
					m_Mobile.Combatant = null;
					m_Mobile.FocusMob = null;
					m_Mobile.CurrentSpeed = m_Mobile.PassiveSpeed;
					break;

				case ActionType.Combat:
					m_Mobile.Warmode = true;
					m_Mobile.FocusMob = null;
					m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
					break;

				case ActionType.Guard:
					m_Mobile.Warmode = true;
					m_Mobile.FocusMob = null;
					m_Mobile.Combatant = null;
					m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
					m_NextStopGuard = DateTime.Now + TimeSpan.FromSeconds( 10 );
					m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
					break;

				case ActionType.Flee:
					m_Mobile.Warmode = true;
					m_Mobile.FocusMob = null;
					m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
					break;

				case ActionType.Interact:
					m_Mobile.Warmode = false;
					m_Mobile.CurrentSpeed = m_Mobile.PassiveSpeed;
					break;

				case ActionType.Backoff:
					m_Mobile.Warmode = false;
					m_Mobile.CurrentSpeed = m_Mobile.PassiveSpeed;
					break;
			}
		}

		public virtual bool DoActionWander()
		{
			WalkRandomInHome(2, 2, 1);

			if ( m_Mobile.Combatant != null && !m_Mobile.Combatant.Deleted )
			{
				m_Mobile.Direction = m_Mobile.GetDirectionTo( m_Mobile.Combatant );                
			}

			return true;
		}

		public virtual bool DoActionCombat()
		{
			Mobile c = m_Mobile.Combatant;

			if ( c == null || c.Deleted || c.Map != m_Mobile.Map )
			{
				Action = ActionType.Wander;				
			}
			else
			{
                m_Mobile.Direction = m_Mobile.GetDirectionTo( c );                
			}
			
			return true;
		}

		public virtual bool DoActionGuard()
		{
			if ( DateTime.Now < m_NextStopGuard )
			{
				m_Mobile.DebugSay("I am on guard");
				m_Mobile.Turn( Utility.Random(0, 2) - 1 );
			}
			else
			{
				m_Mobile.DebugSay("I stop be in Guard");
				Action = ActionType.Wander;
			}

			return true;
		}

		public virtual bool DoActionFlee()
		{
			Mobile from = m_Mobile.FocusMob;

			if ( from == null || from.Deleted || from.Map != m_Mobile.Map )
			{
				m_Mobile.DebugSay("I have lost im");
				Action = ActionType.Guard;
				return true;
			}

			if ( WalkMobileRange( from, 1, true, m_Mobile.RangePerception*2, m_Mobile.RangePerception*3 ) )
			{
				m_Mobile.DebugSay("I Have fled");
				Action = ActionType.Guard;
				return true;                        
			}
			else
			{
				m_Mobile.DebugSay("I am fleeing!");
			}

			return true;
		}

		public virtual bool DoActionInteract()
		{
			return true;
		}

		public virtual bool DoActionBackoff()
		{
			return true;
		}

		public virtual bool Obey()
		{
			if ( m_Mobile.Deleted )
				return false;

			switch ( m_Mobile.ControlOrder )
			{
				case OrderType.None:
					return DoOrderNone();

				case OrderType.Come:
					return DoOrderCome();

				case OrderType.Drop:
					return DoOrderDrop();

				case OrderType.Friend:
					return DoOrderFriend();

				case OrderType.Guard:
					return DoOrderGuard();

				case OrderType.Attack:
					return DoOrderAttack();

				case OrderType.Patrol:
					return DoOrderPatrol();

				case OrderType.Release:
					return DoOrderRelease();

				case OrderType.Stay:
					return DoOrderStay();

				case OrderType.Stop:
					return DoOrderStop();

				case OrderType.Follow:
					return DoOrderFollow();

				case OrderType.Transfert:
					return DoOrderTransfert();

				default:
					return false;
			}
		}

		public virtual void OnCurrentOrderChanged()
		{
			if ( m_Mobile.Deleted || m_Mobile.ControlMaster == null || m_Mobile.ControlMaster.Deleted )
				return;

			switch ( m_Mobile.ControlOrder )
			{
				case OrderType.None:
					m_Mobile.ControlMaster.RevealingAction();
					m_Mobile.Home = m_Mobile.Location;
					m_Mobile.CurrentSpeed = m_Mobile.PassiveSpeed;
					m_Mobile.PlaySound( m_Mobile.GetIdleSound() );
					m_Mobile.Warmode = false;
					m_Mobile.Combatant = null;
					break;

				case OrderType.Come:
					m_Mobile.ControlMaster.RevealingAction();
					m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
					m_Mobile.PlaySound( m_Mobile.GetIdleSound() );
					m_Mobile.Warmode = false;
					m_Mobile.Combatant = null;
					break;

				case OrderType.Drop:
					m_Mobile.ControlMaster.RevealingAction();
					m_Mobile.CurrentSpeed = m_Mobile.PassiveSpeed;
					m_Mobile.PlaySound( m_Mobile.GetIdleSound() );
					m_Mobile.Warmode = true;
					m_Mobile.Combatant = null;
					break;

				case OrderType.Friend:
					m_Mobile.ControlMaster.RevealingAction();
					m_Mobile.WaitForTarget = true;

					break;

				case OrderType.Guard:
					m_Mobile.ControlMaster.RevealingAction();
					m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
					m_Mobile.PlaySound( m_Mobile.GetIdleSound() );
					m_Mobile.Warmode = true;
					m_Mobile.Combatant = null;
					break;

				case OrderType.Attack:
					m_Mobile.ControlMaster.RevealingAction();
					m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
					m_Mobile.WaitForTarget = true;
					m_Mobile.PlaySound( m_Mobile.GetIdleSound() );

					if ( m_Mobile.ControlMaster.Target == null )
					{
						m_Mobile.DebugSay("Create the target");
						m_Mobile.ControlMaster.Target = new AIControlMobileTarget( this );
					}
					else
					{
						if ( m_Mobile.ControlMaster.Target is AIControlMobileTarget )
						{
							m_Mobile.DebugSay("Add my AI to the target");
							((AIControlMobileTarget)m_Mobile.ControlMaster.Target).AddAI( this );
						}
					}

					m_Mobile.Warmode = true;
					m_Mobile.Combatant = null;
					break;

				case OrderType.Patrol:
					m_Mobile.ControlMaster.RevealingAction();
					m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
					m_Mobile.PlaySound( m_Mobile.GetIdleSound() );
					m_Mobile.Warmode = false;
					m_Mobile.Combatant = null;
					break;

				case OrderType.Release:
					m_Mobile.ControlMaster.RevealingAction();
					m_Mobile.CurrentSpeed = m_Mobile.PassiveSpeed;
					m_Mobile.PlaySound( m_Mobile.GetIdleSound() );
					m_Mobile.Warmode = false;
					m_Mobile.Combatant = null;
					break;

				case OrderType.Stay:
					m_Mobile.ControlMaster.RevealingAction();
					m_Mobile.CurrentSpeed = m_Mobile.PassiveSpeed;
					m_Mobile.PlaySound( m_Mobile.GetIdleSound() );
					m_Mobile.Warmode = false;
					m_Mobile.Combatant = null;
					break;

				case OrderType.Stop:
					m_Mobile.ControlMaster.RevealingAction();
					m_Mobile.CurrentSpeed = m_Mobile.PassiveSpeed;
					m_Mobile.PlaySound( m_Mobile.GetIdleSound() );
					m_Mobile.Warmode = false;
					m_Mobile.Combatant = null;
					break;

				case OrderType.Follow:
					m_Mobile.ControlMaster.RevealingAction();
					m_Mobile.CurrentSpeed = m_Mobile.ActiveSpeed;
					m_Mobile.PlaySound( m_Mobile.GetIdleSound() );

					if ( m_AutoTarget == null )
					{
						if ( m_Mobile.ControlMaster.Target == null )
						{
							m_Mobile.DebugSay("Create the target");
							m_Mobile.ControlMaster.Target = new AIControlMobileTarget( this );
						}
						else
						{
							if ( m_Mobile.ControlMaster.Target is AIControlMobileTarget )
							{
								m_Mobile.DebugSay("Add my AI to the target");
								((AIControlMobileTarget)m_Mobile.ControlMaster.Target).AddAI( this );
							}
						}
					}
					else
					{
						m_Mobile.ControlTarget = m_AutoTarget;
						m_Mobile.WaitForTarget = false;
					}

					m_Mobile.Warmode = false;
					m_Mobile.Combatant = null;
					break;

				case OrderType.Transfert:
					m_Mobile.ControlMaster.RevealingAction();
					m_Mobile.CurrentSpeed = m_Mobile.PassiveSpeed;
					m_Mobile.PlaySound( m_Mobile.GetIdleSound() );

					if ( m_Mobile.ControlMaster != null && m_Mobile.ControlMaster.Target == null )
					{
						m_Mobile.DebugSay( "Create the target" );
						m_Mobile.ControlTarget = null;
						m_Mobile.WaitForTarget = true;
						m_Mobile.ControlMaster.Target = new AIControlMobileTarget( this );
					}

					m_Mobile.Warmode = false;
					m_Mobile.Combatant = null;
					break;
			}
		}
		
		public virtual bool DoOrderNone()
		{
			m_Mobile.DebugSay( "I have no order" );

			WalkRandomInHome(3, 2, 1);

			if ( m_Mobile.Combatant != null && !m_Mobile.Combatant.Deleted )
			{
				m_Mobile.Warmode = true;
				m_Mobile.Direction = m_Mobile.GetDirectionTo( m_Mobile.Combatant );                
			}
			else
			{
				m_Mobile.Warmode = false;
			}

			return true;
		}

		public virtual bool DoOrderCome()
		{
			if ( m_Mobile.ControlMaster != null && !m_Mobile.ControlMaster.Deleted )
			{
				int iCurrDist = (int)m_Mobile.GetDistanceToSqrt( m_Mobile.ControlMaster );

				if ( iCurrDist > m_Mobile.RangePerception )
				{
					m_Mobile.DebugSay( "I have lost my master. I stay here" );
					m_Mobile.ControlOrder = OrderType.None;					
				}
				else
				{
					m_Mobile.DebugSay( "My master told me come" );
					
					if ( WalkMobileRange( m_Mobile.ControlMaster, 1, false, 1, 1 ) )
					{
						if ( m_Mobile.Combatant != null && !m_Mobile.Combatant.Deleted )
						{
							m_Mobile.Warmode = true;
							m_Mobile.Direction = m_Mobile.GetDirectionTo( m_Mobile.Combatant );                
						}
						else
						{
							m_Mobile.Warmode = false;
						}
					}
				}
			}

			return true;
		}

		public virtual bool DoOrderDrop()
		{
			if ( m_Mobile.Summoned )
				return true;

			m_Mobile.DebugSay( "I drop my stuff for my master" );

			Container pack = m_Mobile.Backpack;

			if ( pack != null )
			{
				ArrayList list = pack.Items;

				for ( int i = list.Count - 1; i >= 0; --i )
					if ( i < list.Count )
						((Item)list[i]).MoveToWorld( m_Mobile.Location, m_Mobile.Map );
			}

			m_Mobile.ControlOrder = OrderType.None;

			return true;
		}

		public virtual bool DoOrderFollow()
		{
			if (m_Mobile.WaitForTarget)
			{
				m_Mobile.DebugSay( "I am waiting my target" );				
				return true;
			}

			if ( m_Mobile.ControlTarget != null && !m_Mobile.ControlTarget.Deleted && m_Mobile.ControlTarget != m_Mobile )
			{
				int iCurrDist = (int)m_Mobile.GetDistanceToSqrt( m_Mobile.ControlTarget );

				if ( iCurrDist > m_Mobile.RangePerception )
				{
					m_Mobile.DebugSay( "I have lost the one a follow. I stay here" );
					if ( m_Mobile.Combatant != null && !m_Mobile.Combatant.Deleted )
					{
						m_Mobile.Warmode = true;
						m_Mobile.Direction = m_Mobile.GetDirectionTo( m_Mobile.Combatant );                
					}
					else
					{
						m_Mobile.Warmode = false;
					}
				}
				else
				{	
					m_Mobile.DebugSay( "My master told me to follow: {0}", m_Mobile.ControlTarget.Name );
					if ( WalkMobileRange( m_Mobile.ControlTarget, 1, false, 1, 1 ) )
					{
						if ( m_Mobile.Combatant != null && !m_Mobile.Combatant.Deleted )
						{
							m_Mobile.Warmode = true;
							m_Mobile.Direction = m_Mobile.GetDirectionTo( m_Mobile.Combatant );                
						}
						else
						{
							m_Mobile.Warmode = false;
						}
					}
				}
			}
			else
			{
				m_Mobile.DebugSay( "I have nobody to follow, lets relax");
				m_Mobile.ControlOrder = OrderType.None;
			}

			return true;
		}

		public virtual bool DoOrderFriend()
		{
			m_Mobile.DebugSay( "This order is not yet coded" );

			return true;
		}

		public virtual bool DoOrderGuard()
		{
			if ( m_Mobile.ControlMaster == null || m_Mobile.ControlMaster.Deleted )
				return true;

			if ( m_Mobile.ControlMaster.Aggressors.Count > 0 )
			{
				m_Mobile.DebugSay( "My master have been attaqued!" );

				if ( m_Mobile.Combatant == null )
				{
					m_Mobile.DebugSay( "I will attack one of those bastard" );

					for (int i=0; i<m_Mobile.ControlMaster.Aggressors.Count; i++)
					{
						AggressorInfo info = (AggressorInfo)m_Mobile.ControlMaster.Aggressors[i];

						if ( !info.Attacker.Deleted )
						{
							if ( info.Attacker.GetDistanceToSqrt( m_Mobile ) < m_Mobile.RangePerception )
							{
								if ( m_Mobile.Combatant != null )
								{
									if ( info.Attacker.GetDistanceToSqrt( m_Mobile.ControlMaster ) < m_Mobile.Combatant.GetDistanceToSqrt( m_Mobile.ControlMaster ) )
									{
										m_Mobile.Combatant = info.Attacker;
									}
								}
								else
								{
									m_Mobile.Combatant = info.Attacker;
								}
							}
						}
					}

					if ( m_Mobile.Combatant == null )
					{
						m_Mobile.DebugSay( "My master told me to guard im" );
						if ( WalkMobileRange( m_Mobile.ControlMaster, 1, false, 1, 1 ) )
						{
							if ( m_Mobile.Combatant != null && !m_Mobile.Combatant.Deleted )
							{
								m_Mobile.Warmode = true;
								m_Mobile.Direction = m_Mobile.GetDirectionTo( m_Mobile.Combatant );                
							}
							else
							{
								m_Mobile.Warmode = false;
							}
						}
					}
				}
				else
				{
					m_Mobile.DebugSay( "I Fight im!" );				

					m_Mobile.FocusMob = m_Mobile.Combatant;
					DoActionCombat();
				}
			}
			else
			{
				m_Mobile.DebugSay( "My master told me to guard im" );
				if ( WalkMobileRange( m_Mobile.ControlMaster, 1, false, 1, 1 ) )
				{
					if ( m_Mobile.Combatant != null && !m_Mobile.Combatant.Deleted )
					{
						m_Mobile.Warmode = true;
						m_Mobile.Direction = m_Mobile.GetDirectionTo( m_Mobile.Combatant );                
					}
					else
					{
						m_Mobile.Warmode = false;
					}
				}			
			}			
			
			return true;
		}

		public virtual bool DoOrderAttack()
		{
			if (m_Mobile.WaitForTarget)
			{
				m_Mobile.DebugSay( "I am waiting my target" );				
				return true;
			}

			if ( m_Mobile.ControlTarget == null || m_Mobile.ControlTarget.Deleted || m_Mobile.ControlTarget.Map != m_Mobile.Map )
			{
				m_Mobile.DebugSay( "The fight I lost im" );

				m_Mobile.ControlOrder = OrderType.None;
				m_Mobile.ControlTarget = null;
			}
			else
			{
				m_Mobile.DebugSay( "I Fight im!" );				
				Think();
			}

			return true;
		}

		public virtual bool DoOrderPatrol()
		{
			m_Mobile.DebugSay( "This order is not yet coded" );
			return true;
		}

		public virtual bool DoOrderRelease()
		{
			m_Mobile.DebugSay( "My master release me" );

			m_Mobile.PlaySound( m_Mobile.GetAngerSound() );

			m_Mobile.SetControlMaster( null );

			if ( m_Mobile.Summoned )
				m_Mobile.Delete();
			
			return true;
		}

		public virtual bool DoOrderStay()
		{
			if ( m_Mobile.ControlMaster == null || m_Mobile.ControlMaster.Deleted )
				return true;

			m_Mobile.DebugSay( "My master told me to stay" );

			m_Mobile.Direction = m_Mobile.GetDirectionTo( m_Mobile.ControlMaster );

			return true;
		}

		public virtual bool DoOrderStop()
		{
			if ( m_Mobile.ControlMaster == null || m_Mobile.ControlMaster.Deleted )
				return true;

			m_Mobile.DebugSay( "My master told me to stop." );

			m_Mobile.Direction = m_Mobile.GetDirectionTo( m_Mobile.ControlMaster );
			m_Mobile.Home = m_Mobile.Location;

			m_Mobile.ControlOrder = OrderType.None;

			return true;
		}

		public virtual bool DoOrderTransfert()
		{
			if (m_Mobile.WaitForTarget)
			{
				m_Mobile.DebugSay( "I am waiting my target" );				
				return true;
			}

			if ( m_Mobile.ControlTarget != null && !m_Mobile.ControlTarget.Deleted && m_Mobile.ControlTarget.Player )
			{
				int iCurrDist = (int)m_Mobile.GetDistanceToSqrt( m_Mobile.ControlTarget );

				if ( iCurrDist > m_Mobile.RangePerception )
				{
					m_Mobile.DebugSay( "I am too far away to be transfered" );
					if ( m_Mobile.Combatant != null && !m_Mobile.Combatant.Deleted )
					{
						m_Mobile.Warmode = true;
						m_Mobile.Direction = m_Mobile.GetDirectionTo( m_Mobile.Combatant );                
					}
					else
					{
						m_Mobile.Warmode = false;
					}
				}
				else
				{	
					m_Mobile.DebugSay( "My master transfered me to: {0}", m_Mobile.ControlTarget.Name );
					m_Mobile.SetControlMaster( m_Mobile.ControlTarget );
				}
			}
			else
			{
				m_Mobile.DebugSay( "I can't transfer to them, lets relax");
				m_Mobile.ControlOrder = OrderType.None;
			}

			return true;
		}

		public virtual bool DoBardPacified()
		{
			if ( DateTime.Now < m_Mobile.BardEndTime )
			{
				m_Mobile.DebugSay( "I am pacified, I wait" );
				m_Mobile.Combatant = null;
				m_Mobile.Warmode = false;

			}
			else
			{
				m_Mobile.DebugSay( "I no more Pacified" );
				m_Mobile.BardPacified = false;
			}

			return true;
		}

		public virtual bool DoBardProvoked()
		{
			if ( DateTime.Now >= m_Mobile.BardEndTime && (m_Mobile.BardMaster == null || m_Mobile.BardMaster.Deleted || m_Mobile.BardMaster.Map != m_Mobile.Map || m_Mobile.GetDistanceToSqrt( m_Mobile.BardMaster ) > m_Mobile.RangePerception) )
			{
				m_Mobile.DebugSay( "I have lost my provoker" );
				m_Mobile.BardProvoked = false;
				m_Mobile.BardMaster = null;
				m_Mobile.BardTarget = null;
				
				m_Mobile.Combatant = null;
				m_Mobile.Warmode = false;
			}
			else
			{
				if ( m_Mobile.BardTarget == null || m_Mobile.BardTarget.Deleted || m_Mobile.BardTarget.Map != m_Mobile.Map || m_Mobile.GetDistanceToSqrt( m_Mobile.BardTarget ) > m_Mobile.RangePerception )
				{
					m_Mobile.DebugSay( "I have lost my provoke target" );
					m_Mobile.BardProvoked = false;
					m_Mobile.BardMaster = null;
					m_Mobile.BardTarget = null;

					m_Mobile.Combatant = null;
					m_Mobile.Warmode = false;
				}
				else
				{
					Think();

					//m_Mobile.Combatant = m_Mobile.BardTarget;
					//DoActionCombat();
				}
			}

            return true;
		}

		public virtual void WalkRandom(int iChanceToNotMove, int iChanceToDir, int iSteps)
		{
			if ( m_Mobile.Deleted || m_Mobile.DisallowAllMoves )
				return;

			for (int i=0; i<iSteps; i++)
			{
				if ( Utility.Random( 8 * iChanceToNotMove ) <= 8 )
				{
					int iRndMove = Utility.Random(0,8 + ( 9*iChanceToDir ));

					switch (iRndMove)
					{
						case 0:
							DoMove( Direction.Up );
							break;
						case 1:
							DoMove( Direction.North );
							break;
						case 2:
							DoMove( Direction.Left );
							break;
						case 3:
							DoMove( Direction.West );
							break;
						case 5:
							DoMove( Direction.Down );
							break;
						case 6:
							DoMove( Direction.South );
							break;
						case 7:
							DoMove( Direction.Right );
							break;
						case 8:
							DoMove( Direction.East );
							break;
						default:
							DoMove( m_Mobile.Direction );
							break;
					}
				}
			}
		}

		public static double TransformMoveDelay( double delay )
		{
			if ( delay == 0.2 )
				return 0.3;
			else if ( delay == 0.3 )
				return 0.6;
			else if ( delay == 0.4 )
				return 0.9;
			else
				return delay;
		}

		private DateTime m_NextMove;

		public virtual bool CheckMove()
		{
			return ( DateTime.Now >= m_NextMove );
		}

		public virtual bool DoMove( Direction d )
		{
			if ( m_Mobile.Deleted || m_Mobile.Frozen || m_Mobile.Paralyzed || (m_Mobile.Spell != null && m_Mobile.Spell.IsCasting) || m_Mobile.DisallowAllMoves )
				return false;
			else if ( !CheckMove() )
				return false;

			// This makes them always move one step, never any direction changes
			m_Mobile.Direction = d;

			m_NextMove = DateTime.Now + TimeSpan.FromSeconds( TransformMoveDelay( m_Mobile.CurrentSpeed ) );

			if ( (m_Mobile.Direction & Direction.Mask) != (d & Direction.Mask) )
			{
				return m_Mobile.Move( d );
			}
			else if ( !m_Mobile.Move( d ) )
			{
				int offset = (Utility.RandomDouble() >= 0.6 ? 1 : -1);

				for ( int i = 0; i < 2; ++i )
				{
					m_Mobile.Turn( offset );

					if ( m_Mobile.Move( m_Mobile.Direction ) )
						return true;
				}

				return false;
			}

			return true;
		}

		public virtual void WalkRandomInHome(int iChanceToNotMove, int iChanceToDir, int iSteps)
		{
			if ( m_Mobile.Deleted || m_Mobile.DisallowAllMoves )
				return;

			if ( m_Mobile.Home == Point3D.Zero )
			{
				WalkRandom(iChanceToNotMove, iChanceToDir, iSteps);
			}
			else
			{
				for (int i=0; i<iSteps; i++)
				{
					if ( m_Mobile.RangeHome != 0 )
					{
						int iCurrDist = (int)m_Mobile.GetDistanceToSqrt( m_Mobile.Home );

						if ( iCurrDist < m_Mobile.RangeHome * 2 / 3 )
						{
							WalkRandom(iChanceToNotMove, iChanceToDir, 1);
						}
						else if (iCurrDist > m_Mobile.RangeHome)
						{
							DoMove( m_Mobile.GetDirectionTo( m_Mobile.Home ) );
						}
						else
						{
							if ( Utility.Random( 10 ) > 5 )
							{
								DoMove( m_Mobile.GetDirectionTo( m_Mobile.Home ) );
							}
							else
							{
								WalkRandom(iChanceToNotMove, iChanceToDir, 1);
							}
						}
					}
					else
					{
						if ( m_Mobile.Location != m_Mobile.Home )
						{
							DoMove( m_Mobile.GetDirectionTo( m_Mobile.Home ) );
						}
					}
				}
			}
		}

		/*
		 *  Walk at range distance from mobile
		 * 
		 *	iSteps : Number of steps
		 *	bRun   : Do we run
		 *	iWantDistMin : The minimum distance we want to be
		 *  iWantDistMax : The maximum distance we want to be
		 * 
		 */ 
		public virtual bool WalkMobileRange(Mobile m, int iSteps, bool bRun, int iWantDistMin, int iWantDistMax)
		{
			if ( m_Mobile.Deleted || m_Mobile.DisallowAllMoves )
				return false;

			if (m != null)
			{
				for (int i=0; i<iSteps; i++)
				{
					// Get the curent distance
					int iCurrDist = (int)m_Mobile.GetDistanceToSqrt( m );

					if (iCurrDist < iWantDistMin || iCurrDist > iWantDistMax)
					{
						Direction dirTo;

						if ( iCurrDist > iWantDistMax  )
						{
							// Let's get closer
							dirTo = m_Mobile.GetDirectionTo( m );
						}
						else
						{
							dirTo = m.GetDirectionTo( m_Mobile );
						}

						// Add the run flag
						if (bRun)
						{
							dirTo = dirTo | Direction.Running;
						}

						DoMove( dirTo );

						


						/*if ( !m_Mobile.Move( dirTo ) )
						{
							// Here will call a PATH FINDING system
							// For now, we go left or right
							if ( Utility.Random(0,10) >= 5 )
							{
								m_Mobile.DebugSay( "Block: Turn Left" );
								m_Mobile.Turn( 2 );

								if ( !m_Mobile.Move( m_Mobile.Direction ) )
								{
									m_Mobile.DebugSay( "Block the other side" );
									m_Mobile.Turn( 4 );
									m_Mobile.Move( m_Mobile.Direction );
								}
							}
							else
							{
								m_Mobile.DebugSay( "Block: Turn right" );
								m_Mobile.Turn( -2 );

								if ( !m_Mobile.Move( m_Mobile.Direction ) )
								{
									m_Mobile.DebugSay( "Block the other side" );
									m_Mobile.Turn( 4 );
									m_Mobile.Move( m_Mobile.Direction );
								}
							}
						}*/
					}
					else
					{
						return true;
					}
				}

				// Get the curent distance
				int iNewDist = (int)m_Mobile.GetDistanceToSqrt( m );

				if (iNewDist >= iWantDistMin && iNewDist <= iWantDistMax)
				{
					return true;                    
				}
				else
				{
					return false;
				}
			}

			return false;
		}

		/*
		 * Here we check to aquire a target from our surronding
		 * 
		 *  iRange : The range
		 *  acqType : A type of aquire we want (closest, stronguest, etc
		 *  bPlayerOnly : Don't bother with other creature or NPC, want a player
		 *  bFacFriend : Check peoples in my faction
		 *  bFacFoe : Check peoples in other factions
		 * 
		 */ 
		public virtual bool AquireFocusMob(int iRange, FightMode acqType, bool bPlayerOnly, bool bFacFriend, bool bFacFoe)
		{
			if ( m_Mobile.Deleted )
				return false;

			if ( m_Mobile.BardProvoked )
			{
				if ( m_Mobile.BardTarget == null || m_Mobile.BardTarget.Deleted )
				{
					m_Mobile.FocusMob = null;
					return false;
				}
				else
				{
					m_Mobile.FocusMob = m_Mobile.BardTarget;
					return ( m_Mobile.FocusMob != null );
				}
			}
			else if ( m_Mobile.Controled )
			{
				if ( m_Mobile.ControlTarget == null || m_Mobile.ControlTarget.Deleted )
				{
					m_Mobile.FocusMob = null;
					return false;
				}
				else
				{
					m_Mobile.FocusMob = m_Mobile.ControlTarget;
					return ( m_Mobile.FocusMob != null );
				}
			}

			if ( acqType == FightMode.None )
			{
				m_Mobile.FocusMob = null;
				return false;
			}

			if ( acqType == FightMode.Agressor && m_Mobile.Aggressors.Count == 0 )
			{
				m_Mobile.FocusMob = null;
				return false;
			}

			Map map = m_Mobile.Map;

			if ( map != null )
			{
				Mobile newFocusMob = null;
				double val = double.MinValue;

				IPooledEnumerable eable = map.GetMobilesInRange( m_Mobile.Location, iRange);

				foreach ( Mobile m in eable )
				{
					bool bCheckIt = false;

					//
					// Basic check
					//
					if ( ( bPlayerOnly && m.Player ) || !bPlayerOnly )
					{
						if ( m.AccessLevel == AccessLevel.Player && m.Alive && !m.Deleted && m != m_Mobile && m_Mobile.CanSee( m ) )
						{
							bCheckIt = true;
						}
					}

					//
					// Team check
					//
					if (bCheckIt) // alrealy passed the others tests
					{
						if ( m is BaseCreature )
						{
							if (bFacFriend && bFacFoe)
							{
								//check it
							}
                            else if (bFacFriend) // we want allies only
							{
								// but its not a friend
								if ( ((BaseCreature)m).Controled || ((BaseCreature)m).Team != m_Mobile.Team )
								{
                                    bCheckIt = false;		                            
								}
							}
							else if (bFacFoe) // we want foes only
							{
								// but its a friend
								if ( !((BaseCreature)m).Controled && ((BaseCreature)m).Team == m_Mobile.Team )
								{
									bCheckIt = false;		                            
								}
							}
						}
						else // is a player
						{
							// If it's a player, it can't be in your faction
							if ( bFacFriend )
							{
								bCheckIt = false;
							}
						}
					}

					if (bCheckIt)
					{
						if ( acqType == FightMode.Agressor )
						{
							bCheckIt = false;

							for ( int a=0; a < m_Mobile.Aggressors.Count; a++ )
							{
								if ( ((AggressorInfo)m_Mobile.Aggressors[a]).Attacker == m )
								{
									bCheckIt = true;
								}
							}
						}
					}

					if (bCheckIt)
					{
						double theirVal = m_Mobile.GetValueFrom( m , acqType, bPlayerOnly);

						if ( theirVal > val )
						{
							newFocusMob = m;
							val = theirVal;
						}
					}
				}

				eable.Free();

				m_Mobile.FocusMob = newFocusMob;
			}

			return ( m_Mobile.FocusMob != null );
		}

		/*
		 * 
		 *  A Detect Hiden prototype
		 * 
		 */ 
		public virtual void DetectHiden()
		{
			if ( m_Mobile.Deleted )
				return;

			if (m_Mobile.Combatant == null && m_Mobile.FocusMob == null)
			{
				m_Mobile.DebugSay("I check for hiden");
				
				Map map = m_Mobile.Map;

				if ( map != null )
				{
					double srcSkill = m_Mobile.Skills[SkillName.DetectHidden].Value;

					IPooledEnumerable inRange = map.GetMobilesInRange( m_Mobile.Location, m_Mobile.RangePerception);

					foreach ( Mobile trg in inRange )
					{
						if ( trg != m_Mobile && trg.Hidden && trg.AccessLevel < AccessLevel.GameMaster  )
						{
							m_Mobile.DebugSay("I try to detect " + trg.Name);

							double trgSkill = trg.Skills[SkillName.Hiding].Value;

							double x = (srcSkill * (Utility.Random( 25 ) + 75));
							double y = (trgSkill * (Utility.Random( 35 ) + 65));

							if ( x > y )
							{
								trg.Hidden = false;
								trg.SendLocalizedMessage( 500814 ); // You have been revealed!
							}
						}
					}

					inRange.Free();
				}
			}
			else
			{
				m_Mobile.DebugSay("I have no time to check hiden");
			}
		}

		/*
		 *  The mobile changed it speed, we must ajust the timer
		 */ 
		public virtual void OnCurrentSpeedChanged()
		{
			m_Timer.Stop();
			m_Timer.Delay = TimeSpan.FromSeconds( m_Mobile.CurrentSpeed );
			m_Timer.Interval = TimeSpan.FromSeconds( m_Mobile.CurrentSpeed );
			m_Timer.Start();
		}

		private class InternalDelayTimer : Timer
		{
			private Mobile m_Mobile;

			public InternalDelayTimer( Mobile from ) : base( TimeSpan.FromSeconds( 3 ) )
			{
				m_Mobile = from;
			}

			protected override void OnTick()
			{
				m_Mobile.EndAction( typeof( BaseCreature ) );
			}
		}		
		
		/*
		 *  The Timer object
		 */ 
		private class AITimer : Timer
		{
			private BaseAI m_Owner;
			//private DateTime m_NextDetectHidden;
			
			//private bool m_bDetectHidden;

			public AITimer( BaseAI owner ) : base( TimeSpan.FromSeconds( owner.m_Mobile.CurrentSpeed ), TimeSpan.FromSeconds( owner.m_Mobile.CurrentSpeed ) )
			{
				m_Owner = owner;

				//m_bDetectHidden = false;
				//m_NextDetectHidden = DateTime.Now;

				Priority = TimerPriority.FiftyMS;				
			}

			protected override void OnTick()
			{
				if ( m_Owner.m_Mobile.Deleted )
				{
					Stop();
					return;
				}
				else if ( m_Owner.m_Mobile.Map == null || m_Owner.m_Mobile.Map == Map.Internal )
				{
					return;
				}

				if ( m_Owner.m_Mobile.BardPacified )
				{
					m_Owner.DoBardPacified();
				}
				else if ( m_Owner.m_Mobile.BardProvoked )
				{
					m_Owner.DoBardProvoked();
				}
				else
				{
					if ( !m_Owner.m_Mobile.Controled )
					{
						if ( !m_Owner.Think() )
						{
							Stop();
							return;
						}
					}
					else
					{
						if ( !m_Owner.Obey() )
						{
							Stop();
							return;
						}
					}
				}
				//
				// Just that I dont want to do Detect Hiden for now
				// I'll add it later
				//
				/*
				if (m_bDetectHidden && DateTime.Now > m_NextDetectHidden)
				{
					m_Owner.DetectHiden();
					m_NextDetectHidden = DateTime.Now + TimeSpan.FromSeconds( 0.2 * (double)(150-m_Owner.m_Mobile.Int) );
				}
				else
				{
					m_bDetectHidden = m_Owner.m_Mobile.Skills[SkillName.DetectHidden].Value > 0;
				}
				*/
			}
		}
	}
}