using System;
using Server.Gumps;
using Server.Network;
using Server.Items;

namespace Server.Engines.Craft
{
	public class CraftGump : Gump
	{
		private Mobile m_From;
		private CraftSystem m_CraftSystem;
		private int m_GroupCount;
		private int m_SelectedGroup;
		private int m_GroupPage;
		private int m_ItemPage;
		private int m_ResIndex;
		private BaseTool m_Tool;

		private const int LabelHue = 0x480; // 0x384

		public CraftGump( Mobile from, CraftSystem craftSystem, BaseTool tool ): this( from, craftSystem, -1, 1, 1, -1, tool )
		{
		}

		public CraftGump( Mobile from, CraftSystem craftSystem, int selectedGroup, int groupPage, int itemPage, int resIndex, BaseTool tool ) : base( 40, 40 )
		{
			m_From = from;
			m_CraftSystem = craftSystem;
			m_SelectedGroup = selectedGroup;
			m_GroupPage = groupPage;
			m_ItemPage = itemPage;
			m_ResIndex = resIndex;
			m_Tool = tool;

			AddPage( 0 );
			AddBackground( 0, 0, 531, 417, 5054 );
			AddImageTiled( 10, 10, 511, 397, 2624 );
			
			AddImageTiled( 10, 32, 511, 5, 5058 );
			AddImageTiled( 10, 287, 511, 5, 5058 );
			AddImageTiled( 10, 337, 511, 5, 5058 );
			
			AddImageTiled( 210, 37, 5, 250, 5058 );
			AddImageTiled( 160, 292, 5, 45, 5058 );

			AddAlphaRegion( 10, 10, 511, 397 );

			AddLabel( craftSystem.GumpTitlePos, 10, LabelHue, craftSystem.GumpTitle );
			AddLabel( 58, 40, LabelHue, "CATEGORIES" );
			AddLabel( 320, 40, LabelHue, "SELECTION" );
			AddLabel( 48, 295, LabelHue, "NOTICES" );

			AddButton( 15, 382, 0xFB1, 0xFB3, 0, GumpButtonType.Reply, 0 );
			AddLabel( 48, 382, LabelHue, "EXIT" );

			m_GroupCount = CreateGroupList( );

			if ( selectedGroup > -1)
			{
				CreateItemList( selectedGroup );
			}

			// If the system has more than one resource
			if ( craftSystem.CraftSubRes.Init )
			{
				AddButton( 15, 362, 0xFA5, 0xFA7, 5000, GumpButtonType.Reply, 0 );
				string nameRes = craftSystem.CraftSubRes.ResName;

				if ( resIndex > -1 )
				{
					nameRes = craftSystem.CraftSubRes.GetAt( resIndex ).ItemName;
				}

				AddLabel( 48, 362, LabelHue, nameRes );
			}
			// ****************************************

			// Resmelt option
			if ( craftSystem.Resmelt )
			{
				AddButton( 415, 362, 0xFA5, 0xFA7, 5001, GumpButtonType.Reply, 0 );
				AddLabel( 448, 362, LabelHue, "Resmelt" );
			}
			// ****************************************
		}

		public void CreateResList()
		{
			int iCount = m_CraftSystem.CraftSubRes.Count;
			if ( iCount > 10 * m_ItemPage)
			{
				iCount = 10 * m_ItemPage;

				// Draw the Next Button since they are some more items hidden
				AddButton(495, 42, 0x15E1, 0x15E5, 4001, GumpButtonType.Reply, 0 );
			}
			
			if (m_ItemPage > 1)
			{
				// Draw the Previous Button since we'r not at the page 1
				AddButton( 478, 42, 0x15E3, 0x15E7, 4000, GumpButtonType.Reply, 0 );
			}
			
			int j = 0;
			for ( int i = 0 + ( ( 10 * m_ItemPage ) - 10 ); i < iCount; i++ )
			{
				AddButton( 220, ( 20 * j ) + 65, 0xFA5, 0xFA7, i + 6001, GumpButtonType.Reply, 0 );
				
				AddLabel( 253, ( 20 * j ) + 65, LabelHue, m_CraftSystem.CraftSubRes.GetAt( i ).ItemName );
				j++;
			}
		}

		public void CreateItemList( int selectedGroup )
		{
			// 500 means that we want Resource Selection Page
			if ( selectedGroup == 500 )
			{
				CreateResList();
				return;			
			}

			CraftGroupCol craftGroupCol = m_CraftSystem.CraftGroups;
			CraftGroup craftGroup = craftGroupCol.GetAt( selectedGroup );
			CraftItemCol craftItemCol = craftGroup.CraftItems;

			int iCount = craftItemCol.Count;
			if ( iCount > 10 * m_ItemPage)
			{
				iCount = 10 * m_ItemPage;

				// Draw the Next Button since they are some more items hidden
				AddButton(495, 42, 0x15E1, 0x15E5, 4001, GumpButtonType.Reply, 0 );
			}
			
			if (m_ItemPage > 1)
			{
				// Draw the Previous Button since we'r not at the page 1
				AddButton( 478, 42, 0x15E3, 0x15E7, 4000, GumpButtonType.Reply, 0 );
			}

			int j = 0;
			for ( int i = 0 + ( ( 10 * m_ItemPage ) - 10 ); i < iCount; i++ )
			{
				AddButton( 220, ( 20 * j ) + 65, 0xFA5, 0xFA7, i + 1001, GumpButtonType.Reply, 0 );
				AddButton( 480, ( 20 * j ) + 65, 0xFAB, 0xFAD, i + 2001, GumpButtonType.Reply, 0 );
				AddLabel( 253, ( 20 * j ) + 65, LabelHue, craftItemCol.GetAt( i ).ItemName );
				j++;
			}

		}

		public int CreateGroupList()
		{
			CraftGroupCol craftGroupCol = m_CraftSystem.CraftGroups;

			for ( int i = 0; i < craftGroupCol.Count; i++ )
			{
				AddButton(15, (20 * i) + 65, 0xFA5, 0xFA7, i + 1, GumpButtonType.Reply, 0 );
				AddLabel(48, (20 * i) + 65, LabelHue, craftGroupCol.GetAt( i ).GroupName);
			}

			return craftGroupCol.Count;
		}

		public override void OnResponse( NetState sender, RelayInfo info )
		{
			// Previous Button
			if ( info.ButtonID == 4000 )
			{
				m_ItemPage--;
				CraftGump craftGump = new CraftGump( m_From, m_CraftSystem, m_SelectedGroup, m_GroupPage, m_ItemPage, m_ResIndex, m_Tool );
				m_From.SendGump( craftGump );
				return;
			}
			
			// Next Button
			if ( info.ButtonID == 4001 )
			{
				m_ItemPage++;
				CraftGump craftGump = new CraftGump( m_From, m_CraftSystem, m_SelectedGroup, m_GroupPage, m_ItemPage, m_ResIndex, m_Tool );
				m_From.SendGump( craftGump );
				return;
			}

			// Show the Group
			if ( info.ButtonID > 0 && info.ButtonID <= m_GroupCount)
			{
				m_ItemPage = 1;
				CraftGump craftGump = new CraftGump( m_From, m_CraftSystem, info.ButtonID - 1, m_GroupPage, m_ItemPage, m_ResIndex, m_Tool );
				m_From.SendGump( craftGump );
				return;
			}
			
			// Create the selected Item
			if ( info.ButtonID > 1000 && info.ButtonID < 2000)
			{
				CraftGroupCol craftGroupCol = m_CraftSystem.CraftGroups;
				CraftGroup craftGroup = craftGroupCol.GetAt( m_SelectedGroup );
				CraftItemCol craftItemCol = craftGroup.CraftItems;

				if ( m_CraftSystem.CanCraft( m_From, m_Tool, craftItemCol.GetAt( info.ButtonID - 1001 ).ItemType ) )
				{
					Type type = null;
					if ( m_ResIndex > -1 )
					{
						type = m_CraftSystem.CraftSubRes.GetAt( m_ResIndex ).ItemType;
					}

					m_CraftSystem.CreateItem( m_From, craftItemCol.GetAt( info.ButtonID - 1001 ).ItemType, type, m_Tool, craftItemCol.GetAt( info.ButtonID - 1001 ) );
				}

				CraftGump craftGump = new CraftGump( m_From, m_CraftSystem, m_SelectedGroup, m_GroupPage, m_ItemPage, m_ResIndex, m_Tool );
				m_From.SendGump( craftGump );
				return;
			}

			// Show the Item Information
			if ( info.ButtonID > 2000 && info.ButtonID < 3000)
			{
				CraftGroupCol craftGroupCol = m_CraftSystem.CraftGroups;
				CraftGroup craftGroup = craftGroupCol.GetAt( m_SelectedGroup );
				CraftItemCol craftItemCol = craftGroup.CraftItems;

				CraftGumpItem craftGumpItem = new CraftGumpItem( m_From, m_CraftSystem, craftItemCol.GetAt( info.ButtonID - 2001 ), m_SelectedGroup, m_GroupPage, m_ItemPage, m_ResIndex, m_Tool );
				m_From.SendGump( craftGumpItem );
				return;
			}

			// Res Selection
			if ( info.ButtonID == 5000 )
			{
				m_ItemPage = 1;
				CraftGump craftGump = new CraftGump( m_From, m_CraftSystem, 500, m_GroupPage, m_ItemPage, m_ResIndex, m_Tool );
				m_From.SendGump( craftGump );
				return;
			}

			// Resmelt
			if ( info.ButtonID == 5001 )
			{
				Resmelt.Do( m_From );
				return;
			}
			
			// User has choice a resource to work with
			if ( info.ButtonID > 6000 && info.ButtonID < 7000 )
			{
				CraftGump craftGump = new CraftGump( m_From, m_CraftSystem, -1, 1, 1, info.ButtonID - 6001, m_Tool );
				m_From.SendGump( craftGump );
				return;
			}
		}
	}
}