using System;
using Server.Network;
using Server.Items;

namespace Server.Items
{
	public abstract class LockableContainer : TrapableContainer, ILockable, ILockpickable//, ITelekinesisable
	{
		private bool m_Locked;
		private int m_LockLevel;
		private uint m_KeyValue;
		private Mobile m_Picker;


		[CommandProperty( AccessLevel.GameMaster )]
		public Mobile Picker
		{
			get
			{
				return m_Picker;
			}
			set
			{
				m_Picker = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int LockLevel
		{
			get
			{
				return m_LockLevel;
			}
			set
			{
				m_LockLevel = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public bool Locked
		{
			get
			{
				return m_Locked;
			}
			set
			{
				m_Locked = value;
				if ( m_Locked )
					m_Picker = null;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public uint KeyValue
		{
			get
			{
				return m_KeyValue;
			}
			set
			{
				m_KeyValue = value;
			}
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 2 ); // version

			writer.Write( m_KeyValue );
			writer.Write( (int) m_LockLevel );
			writer.Write( (bool) m_Locked );

		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 2:
				{
					m_KeyValue = reader.ReadUInt();

					goto case 1;
				}
				case 1:
				{
					m_LockLevel = reader.ReadInt();

					goto case 0;
				}
				case 0:
				{
					m_Locked = reader.ReadBool();

					break;
				}
			}
		}

		public virtual bool CreateKey{ get{ return true; } }

		public LockableContainer( int itemID, int gumpID, int defaultDropSound, Rectangle2D bounds ) : base( itemID, gumpID, defaultDropSound, bounds )
		{
			if ( CreateKey )
			{
				uint val = (uint)(0xFFFFFFFF * Utility.RandomDouble());
				Key key = new Key( KeyType.Copper, val );

				key.Location = new Point3D( 100, 100, 0 );
				m_KeyValue = val;

				AddItem( key );
			}
		}

		public LockableContainer( Serial serial ) : base( serial )
		{
		}

		public override void OnTelekinesis( Mobile from )
		{
			if ( m_Locked )
			{
				int number;

				if ( from.AccessLevel >= AccessLevel.GameMaster )
				{
					number = 502502; // That is locked, but you open it with your godly powers.
					base.OnTelekinesis( from );
				}
				else
				{
					number = 502503; // That is locked.

					Effects.SendLocationParticles( EffectItem.Create( Location, Map, TimeSpan.FromSeconds( 5.0 ) ), 0x376A, 9, 32, 5022 );
					Effects.PlaySound( Location, Map, 0x1F5 );
				}

				from.Send( new MessageLocalized( Serial, ItemID, MessageType.Regular, 0x3B2, 3, number, "", "" ) );
			}
			else
			{
				base.OnTelekinesis( from );
			}
		}

		public override void OnDoubleClick( Mobile from )
		{
			if ( m_Locked )
			{
				int number;

				if ( from.AccessLevel >= AccessLevel.GameMaster )
				{
					number = 502502; // That is locked, but you open it with your godly powers.
					base.OnDoubleClick( from );
				}
				else
				{
					number = 502503; // That is locked.
				}

				from.Send( new MessageLocalized( Serial, ItemID, MessageType.Regular, 0x3B2, 3, number, "", "" ) );
			}
			else
			{
				base.OnDoubleClick( from );
			}
		}

		public virtual void LockPick( Mobile from )
		{
			Locked = false;
			Picker = from;
		}
	}
}