using System;
using System.Collections;
using Server;

namespace Server.Items
{
	public abstract class BaseInstrument : Item
	{
		private int m_WellSound, m_BadlySound;

		private static Hashtable m_Instruments = new Hashtable();

		public static BaseInstrument GetInstrument( Mobile from )
		{
			BaseInstrument item = m_Instruments[from] as BaseInstrument;

			if ( item == null )
				return null;

			if ( !item.IsChildOf( from ) )
			{
				m_Instruments[from] = null;
				return null;
			}

			return item;
		}
		
		/* Difficulty TODO:
			Add another 100 points for each of the following abilities: 
				Spell Casting 
				Fire Breath 
				Radiation or Aura Damage (Heat, Cold etc.) 
				Resistance to Poison 
				Lifeforce Draining 
				Summoning Undead 

			Add another 20 points for each level of poison attack the creature can do. This ranges from 20 for a level 1 poison attack to 100 for a level 5 poison attack 
		*/
		public static double GetDifficultyFor( Mobile targ )
		{
			double val = targ.Hits + targ.Stam + targ.Mana + targ.SkillsTotal;

			//replace this with the todos from above
			if ( targ.Skills[SkillName.Magery].Value >= 75.0 )
				val += 150;

			if ( val > 700 )
				val = 700.0 + ( ( val - 700 ) / 3.66667 );

			return val / 10.0;
		}

		public static void SetInstrument( Mobile from, BaseInstrument item )
		{
			m_Instruments[from] = item;
		}

		public BaseInstrument( int itemID, int wellSound, int badlySound ) : base( itemID )
		{
			m_WellSound = wellSound;
			m_BadlySound = badlySound;
		}

		public BaseInstrument( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version

			writer.Write( m_WellSound );
			writer.Write( m_BadlySound );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 0:
				{
					m_WellSound = reader.ReadInt();
					m_BadlySound = reader.ReadInt();

					break;
				}
			}
		}

		public override void OnDoubleClick( Mobile from )
		{
			if ( from.BeginAction( typeof( BaseInstrument ) ) )
			{
				SetInstrument( from, this );

				// Delay of 7 second before beign able to play another instrument again
				new InternalTimer( from ).Start();

				if ( from.CheckSkill( SkillName.Musicianship, 0.0, 100.0 ) )
				{
					PlayInstrumentWell( from );
				} 
				else 
				{
					PlayInstrumentBadly( from );
				}
			}
			else
			{
				from.SendLocalizedMessage( 500119 ); // You must wait to perform another action
			}
		}

		public void PlayInstrumentWell( Mobile from )
		{
			from.PlaySound( m_WellSound );
		}

		public void PlayInstrumentBadly( Mobile from )
		{
			from.PlaySound( m_BadlySound );
		}

		private class InternalTimer : Timer
		{
			private Mobile m_From;

			public InternalTimer( Mobile from ) : base( TimeSpan.FromSeconds( 6.0 ) )
			{
				m_From = from;
			}

			protected override void OnTick()
			{
				m_From.EndAction( typeof( BaseInstrument ) );
			}
		}

	}
}