using System;
using Server;
using Server.Items;
using Server.GenerateRandomMagicItems;

namespace Server.Mobiles
{
	public class Silvani : BaseCreature
	{
		[Constructable]
		public Silvani() : base( AIType.AI_Mage, FightMode.Closest, 18, 1, 0.1, 0.2 )
		{
			Body = 176;
			Name = "Silvani";

			BaseSoundID = 0x467;

			SetFameLevel( 5 );
			SetKarmaLevel( 5 );

			SetStr( 304, 357 );
			SetDex( 456, 479 );
			SetInt( 620, 659 );

			VirtualArmor = 50;

			SetSkill( SkillName.Wrestling, 98, 99 );
			SetSkill( SkillName.Tactics, 98, 100 );
			SetSkill( SkillName.MagicResist, 117, 132 );
			SetSkill( SkillName.Magery, 98, 103 );
			SetSkill( SkillName.EvalInt, 100, 100 );
			SetSkill( SkillName.Meditation, 100, 100 );

			PackItem( new Gold( 2000, 3000 ) );

			PackItem( RandomMagicItem.RandomMagicWeapon( (BaseWeapon)RandomMagicItem.RandomWeapon, 3, 5 ) );
			PackItem( RandomMagicItem.RandomMagicWeapon( (BaseWeapon)RandomMagicItem.RandomWeapon, 5, 5 ) );
			PackItem( RandomScroll.CreateScroll( 3, 8 ) );
			PackItem( RandomScroll.CreateScroll( 3, 8 ) );

			switch ( Utility.Random( 9 ) )
			{
				case 0: PackItem( new Amber() ); break;
				case 1: PackItem( new Amethyst() ); break;
				case 2: PackItem( new Citrine() ); break;
				case 3: PackItem( new Diamond() ); break;
				case 4: PackItem( new Emerald() ); break;
				case 5: PackItem( new Ruby() ); break;
				case 6: PackItem( new Sapphire() ); break;
				case 7: PackItem( new StarSapphire() ); break;
				case 8: PackItem( new Tourmaline() ); break;
			}

			switch ( Utility.Random( 9 ) )
			{
				case 0: PackItem( new Amber() ); break;
				case 1: PackItem( new Amethyst() ); break;
				case 2: PackItem( new Citrine() ); break;
				case 3: PackItem( new Diamond() ); break;
				case 4: PackItem( new Emerald() ); break;
				case 5: PackItem( new Ruby() ); break;
				case 6: PackItem( new Sapphire() ); break;
				case 7: PackItem( new StarSapphire() ); break;
				case 8: PackItem( new Tourmaline() ); break;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public override int HitsMax{ get{ return 1000; } }

		public void SpawnPixies( Mobile target )
		{
			Map map = this.Map;

			if ( map == null )
				return;

			int newPixies = Utility.RandomMinMax( 3, 6 );

			for ( int i = 0; i < newPixies; ++i )
			{
				Pixie pixie = new Pixie();

				pixie.Team = this.Team;
				pixie.FightMode = FightMode.Closest;
				pixie.Map = map;

				bool validLocation = false;

				for ( int j = 0; !validLocation && j < 10; ++j )
				{
					int x = X + Utility.Random( 3 ) - 1;
					int y = Y + Utility.Random( 3 ) - 1;
					int z = map.GetAverageZ( x, y );

					if ( validLocation = map.CanFit( x, y, this.Z, 16, false, false ) )
						pixie.Location = new Point3D( x, y, Z );
					else if ( validLocation = map.CanFit( x, y, z, 16, false, false ) )
						pixie.Location = new Point3D( x, y, z );
				}

				if ( !validLocation )
					pixie.Location = this.Location;

				pixie.Combatant = target;
			}
		}

		public override void AlterDamageScalarFrom( Mobile caster, ref double scalar )
		{
			if ( 0.1 >= Utility.RandomDouble() )
				SpawnPixies( caster );
		}

		public override void OnGaveMeleeAttack( Mobile defender )
		{
			defender.Damage( Utility.Random( 20, 10 ) );
			defender.Stam -= Utility.Random( 20, 10 );
			defender.Mana -= Utility.Random( 20, 10 );
		}

		public override void OnGotMeleeAttack( Mobile attacker )
		{
			if ( 0.1 >= Utility.RandomDouble() )
				SpawnPixies( attacker );
		}

		public Silvani( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}
}