using System;
using Server.Targeting;
using Server.Network;
using Server.Mobiles;

namespace Server.SkillHandlers
{
	public class AnimalTaming
	{
		public static void Initialize()
		{
			SkillInfo.Table[(int)SkillName.AnimalTaming].Callback = new SkillUseCallback( OnUse );
		}

		public static TimeSpan OnUse( Mobile m )
		{
			m.RevealingAction();

			m.Target = new InternalTarget();
			m.RevealingAction();

			m.SendLocalizedMessage( 502789 ); // Tame which animal?

			return TimeSpan.Zero;
		}

		private class InternalTarget : Target
		{
			public InternalTarget() :  base ( 6, false, TargetFlags.None )
			{
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				from.RevealingAction();

				if ( targeted is Mobile )
				{
					if ( targeted is BaseCreature )
					{
						BaseCreature creature = (BaseCreature)targeted;

						if ( !creature.Tamable )
						{
							from.SendLocalizedMessage( 502469 ); // That being can not be tamed.
						}
						else if ( creature.Controled )
						{
							from.SendLocalizedMessage( 502467 ); // That animal looks tame already.
						}
						else if ( from.Followers + creature.ControlSlots > from.FollowersMax )
						{
							from.SendLocalizedMessage( 1049611 ); // You have too many followers to tame that creature.
						}
						else if ( from.Skills[SkillName.AnimalTaming].Value >= creature.MinTameSkill )
						{
							// Face the creature
							from.Direction = from.GetDirectionTo( creature );

							from.LocalOverheadMessage( MessageType.Emote, from.EmoteHue, 1010597 ); // You start to tame the creature.
							from.NonlocalOverheadMessage( MessageType.Emote, from.EmoteHue, 1010598 ); // *begins taming a creature.*

							new InternalTimer( from, creature, Utility.Random( 3, 2 ) ).Start();
						}
						else
						{
							from.SendLocalizedMessage( 502806 ); // You have no chance of taming this creature.
						}
					}
					else
					{
						from.SendLocalizedMessage( 502469 ); // That being can not be tamed.
					}
				}
				else
				{
					from.SendLocalizedMessage( 502801 ); // You can't tame that!
				}
			}

			private class InternalTimer : Timer
			{
				private Mobile m_Tamer;
				private BaseCreature m_Creature;
				private int m_MaxCount;
				private int m_Count;


				public InternalTimer( Mobile tamer, BaseCreature creature, int count ) : base( TimeSpan.FromSeconds( 3.0 ), TimeSpan.FromSeconds( 3.0 ), count )
				{
					m_Tamer = tamer;
					m_Creature = creature;
					m_MaxCount = count;
				}

				protected override void OnTick()
				{
					m_Count++;

					if ( !m_Tamer.InRange( m_Creature, 6 ) )
					{
						m_Tamer.SendLocalizedMessage( 502795 ); // You are too far away to continue taming.
						Stop();
					}
					else if ( !m_Tamer.CheckAlive() )
					{
						m_Tamer.SendLocalizedMessage( 502796 ); // You are dead, and cannot continue taming.
						Stop();
					}
					else if ( !m_Tamer.CanSee( m_Creature ) )
					{
						m_Tamer.SendLocalizedMessage( 502800 ); // You can't see that.
						Stop();
					}
					else if ( !m_Creature.Tamable )
					{
						m_Tamer.SendLocalizedMessage( 502469 ); // That being can not be tamed.
						Stop();
					}
					else if ( m_Creature.Controled )
					{
						m_Tamer.SendLocalizedMessage( 502804 ); // That animal looks tame already.
						Stop();
					}
					else if ( m_Count < m_MaxCount )
					{
						m_Tamer.Say( Utility.Random( 502790, 4 ) );
						m_Tamer.Direction = m_Tamer.GetDirectionTo( m_Creature );
					}
					else
					{
						if ( m_Tamer.CheckSkill( SkillName.AnimalTaming, m_Creature.MinTameSkill, m_Creature.MaxTameSkill ) )
						{
							if ( m_Tamer.Skills[SkillName.AnimalTaming].Value >= m_Creature.MaxTameSkill )
								m_Tamer.SendLocalizedMessage( 502797 ); // That wasn't even challenging.
							else
								m_Tamer.SendLocalizedMessage( 502799 ); // It seems to accept you as master.

							m_Creature.SetControlMaster( m_Tamer );
						}
						else
						{
							m_Tamer.SendLocalizedMessage( 502798 ); // You fail to tame the creature.
						}
					}
				}
			}
		}
	}
}