using System;
using Server.Items;
using Server.Targeting;

namespace Server.SkillHandlers
{
	public class Discordance
	{
		public static void Initialize()
		{
			SkillInfo.Table[(int)SkillName.Discordance].Callback = new SkillUseCallback( OnUse );
		}

		public static TimeSpan OnUse( Mobile m )
		{
			m.RevealingAction();

			BaseInstrument instrument = BaseInstrument.GetInstrument( m );

			if ( instrument == null )
			{
				m.Target = new InternalMusicTarget();
				m.SendLocalizedMessage( 500617 ); // What instrument shall you play?
			}
			else
			{
				m.SendLocalizedMessage( 1049541 );// Select your target for your song of discordance
				m.Target = new DiscordanceTarget( instrument );
			}

			return TimeSpan.FromSeconds( 1.0 );
		}

		public class DiscordanceTarget : Target
		{
			private BaseInstrument m_Instrument;

			public DiscordanceTarget( BaseInstrument inst ) : base( 10, false, TargetFlags.Harmful )
			{
				m_Instrument = inst;
			}

			protected override void OnTarget( Mobile from, object target )
			{
				if ( target is Mobile )
				{
					Mobile targ = (Mobile)target;
					if ( !targ.Player )
					{
						double drop = ( from.Skills[SkillName.Discordance].Value / -5.0 ) / 100.0;
						TimeSpan len = TimeSpan.FromSeconds( from.Skills[SkillName.Discordance].Value * 2 );
						double diff = BaseInstrument.GetDifficultyFor( targ );

						from.RevealingAction();
						if ( from.CheckSkill( SkillName.Discordance, diff-25.0, diff+25.0 ) )
						{
							if ( targ.GetStatMod( "DiscordanceStr" ) == null )
							{
								targ.AddStatMod( new StatMod( StatType.Str, "DiscordanceStr", (int)(targ.RawStr*drop), len ) );
								targ.AddStatMod( new StatMod( StatType.Int, "DiscordanceInt", (int)(targ.RawInt*drop), len ) );
								targ.AddStatMod( new StatMod( StatType.Dex, "DiscordanceDex", (int)(targ.RawDex*drop), len ) );

								for (int i=0;i<targ.Skills.Length;i++)
								{
									if ( targ.Skills[i].Value >= 50.0 )
										targ.AddSkillMod( new TimedSkillMod( (SkillName)i, true, targ.Skills[i].Value*drop, len ) );
								}
								from.SendLocalizedMessage( 1049539 );// You play the song surpressing your targets strength
								m_Instrument.PlayInstrumentWell( from );
							}
							else
							{
								from.SendLocalizedMessage( 1049537 );// Your target is already in discord.
							}
						}
						else
						{
							from.SendLocalizedMessage( 1049540 );// You fail to disrupt your target
							m_Instrument.PlayInstrumentBadly( from );
						}
					}
					else
					{
						m_Instrument.PlayInstrumentBadly( from );
					}
				}
			}
		}

		private class InternalMusicTarget : Target
		{
			public InternalMusicTarget() : base ( 1, false, TargetFlags.None )
			{
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is BaseInstrument )
				{
					BaseInstrument.SetInstrument( from, (BaseInstrument)targeted );

					from.SendLocalizedMessage( 1049541 );// Select your target for your song of discordance
					from.Target = new DiscordanceTarget( (BaseInstrument)targeted );
				}
				else
				{
					from.SendLocalizedMessage( 500619 ); // That is not a musical instrument.
				}
			}
		}
	}
}
