using System;
using Server;

namespace Server.Spells
{
	public class SpellInfo
	{
		private string m_Name;
		private string m_Mantra;
		private SpellCircle m_Circle;
		private Type[] m_Reagents;
		private int[] m_Amounts;
		private int m_Action;
		private bool m_AllowTown;
		private int m_LeftHandEffect, m_RightHandEffect;

		public SpellInfo( string name, string mantra, SpellCircle circle, params Type[] regs ) : this( name, mantra, circle, 16, 0, 0, true, regs )
		{
		}

		public SpellInfo( string name, string mantra, SpellCircle circle, bool allowTown, params Type[] regs ) : this( name, mantra, circle, 16, 0, 0, allowTown, regs )
		{
		}

		public SpellInfo( string name, string mantra, SpellCircle circle, int action, params Type[] regs ) : this( name, mantra, circle, action, 0, 0, true, regs )
		{
		}

		public SpellInfo( string name, string mantra, SpellCircle circle, int action, bool allowTown, params Type[] regs ) : this( name, mantra, circle, action, 0, 0, allowTown, regs )
		{
		}

		public SpellInfo( string name, string mantra, SpellCircle circle, int action, int handEffect, params Type[] regs ) : this( name, mantra, circle, action, handEffect, handEffect, true, regs )
		{
		}

		public SpellInfo( string name, string mantra, SpellCircle circle, int action, int handEffect, bool allowTown, params Type[] regs ) : this( name, mantra, circle, action, handEffect, handEffect, allowTown, regs )
		{
		}

		public SpellInfo( string name, string mantra, SpellCircle circle, int action, int leftHandEffect, int rightHandEffect, bool allowTown, params Type[] regs )
		{
			m_Name = name;
			m_Mantra = mantra;
			m_Circle = circle;
			m_Action = action;
			m_Reagents = regs;
			m_AllowTown = allowTown;

			m_LeftHandEffect = leftHandEffect;
			m_RightHandEffect = rightHandEffect;

			m_Amounts = new int[regs.Length];

			for ( int i = 0; i < regs.Length; ++i )
				m_Amounts[i] = 1;
		}

		public int Action{ get{ return m_Action; } }
		public bool AllowTown{ get{ return m_AllowTown; } }
		public int[] Amounts{ get{ return m_Amounts; } }
		public SpellCircle Circle{ get{ return m_Circle; } }
		public string Mantra{ get{ return m_Mantra; } }
		public string Name{ get{ return m_Name; } }
		public Type[] Reagents{ get{ return m_Reagents; } }
		public int LeftHandEffect{ get{ return m_LeftHandEffect; } }
		public int RightHandEffect{ get{ return m_RightHandEffect; } }
	}
}